@@ -33,19 +33,15 @@
 #include <sys/socket.h>
 #include <sys/un.h>
 
-#include <bluetooth/bluetooth.h>
-#include <bluetooth/hci.h>
+#include "lib/bluetooth.h"
+#include "lib/hci.h"
 
-#include "monitor/mainloop.h"
+#include "src/shared/util.h"
+#include "src/shared/mainloop.h"
 #include "monitor/bt.h"
 
 #include "amp.h"
 
-#define le16_to_cpu(val) (val)
-#define le32_to_cpu(val) (val)
-#define cpu_to_le16(val) (val)
-#define cpu_to_le32(val) (val)
-
 #define PHY_MODE_IDLE		0x00
 #define PHY_MODE_INITIATOR	0x01
 #define PHY_MODE_ACCEPTOR	0x02
