@@ -250,6 +250,7 @@ static gboolean req_timeout(gpointer user_data)
 
 	g_assert(p != NULL);
 
+	p->timeout_id = 0;
 	obex->pending_req = NULL;
 
 	err = g_error_new(G_OBEX_ERROR, G_OBEX_ERROR_TIMEOUT,
@@ -513,7 +514,7 @@ static void enable_tx(GObex *obex)
 	if (obex->suspended)
 		return;
 
-	if (obex->write_source > 0)
+	if (!obex->io || obex->write_source > 0)
 		return;
 
 	cond = G_IO_OUT | G_IO_HUP | G_IO_ERR | G_IO_NVAL;
@@ -525,6 +526,8 @@ static gboolean g_obex_send_internal(GObex *obex, struct pending_pkt *p,
 {
 
 	if (obex->io == NULL) {
+		if (!err)
+			return FALSE;
 		g_set_error(err, G_OBEX_ERROR, G_OBEX_ERROR_DISCONNECTED,
 					"The transport is not connected");
 		g_obex_debug(G_OBEX_DEBUG_ERROR, "%s", (*err)->message);
@@ -662,6 +665,8 @@ gboolean g_obex_send(GObex *obex, GObexPacket *pkt, GError **err)
 	g_obex_debug(G_OBEX_DEBUG_COMMAND, "conn %u", obex->conn_id);
 
 	if (obex == NULL || pkt == NULL) {
+		if (!err)
+			return FALSE;
 		g_set_error(err, G_OBEX_ERROR, G_OBEX_ERROR_INVALID_ARGS,
 				"Invalid arguments");
 		g_obex_debug(G_OBEX_DEBUG_ERROR, "%s", (*err)->message);
@@ -778,7 +783,9 @@ static gboolean pending_req_abort(GObex *obex, GError **err)
 
 	p->cancelled = TRUE;
 
-	g_source_remove(p->timeout_id);
+	if (p->timeout_id > 0)
+		g_source_remove(p->timeout_id);
+
 	p->timeout = G_OBEX_ABORT_TIMEOUT;
 	p->timeout_id = g_timeout_add_seconds(p->timeout, req_timeout, obex);
 
@@ -846,7 +853,7 @@ immediate_completion:
 }
 
 gboolean g_obex_send_rsp(GObex *obex, guint8 rspcode, GError **err,
-						guint8 first_hdr_type, ...)
+						guint first_hdr_type, ...)
 {
 	GObexPacket *rsp;
 	va_list args;
@@ -922,7 +929,11 @@ static void g_obex_srm_suspend(GObex *obex)
 	struct pending_pkt *p = obex->pending_req;
 	GObexPacket *req;
 
-	g_source_remove(p->timeout_id);
+	if (p->timeout_id > 0) {
+		g_source_remove(p->timeout_id);
+		p->timeout_id = 0;
+	}
+
 	p->suspended = TRUE;
 
 	req = g_obex_packet_new(G_OBEX_OP_GET, TRUE,
@@ -1017,8 +1028,10 @@ static void auth_challenge(GObex *obex)
 
 	/* Remove it as pending and add it back to the queue so it gets sent
 	 * again */
-	g_source_remove(p->timeout_id);
-	p->timeout_id = 0;
+	if (p->timeout_id > 0) {
+		g_source_remove(p->timeout_id);
+		p->timeout_id = 0;
+	}
 	obex->pending_req = NULL;
 	g_obex_send_internal(obex, p, NULL);
 }
@@ -1076,7 +1089,9 @@ static gboolean parse_response(GObex *obex, GObexPacket *rsp)
 	 * continue sending responses until the transfer is finished
 	 */
 	if (opcode == G_OBEX_OP_GET && rspcode == G_OBEX_RSP_CONTINUE) {
-		g_source_remove(p->timeout_id);
+		if (p->timeout_id > 0)
+			g_source_remove(p->timeout_id);
+
 		p->timeout_id = g_timeout_add_seconds(p->timeout, req_timeout,
 									obex);
 		return FALSE;
@@ -1219,6 +1234,8 @@ static gboolean read_stream(GObex *obex, GError **err)
 	obex->rx_pkt_len = g_ntohs(u16);
 
 	if (obex->rx_pkt_len > obex->rx_mtu) {
+		if (!err)
+			return FALSE;
 		g_set_error(err, G_OBEX_ERROR, G_OBEX_ERROR_PARSE_ERROR,
 				"Too big incoming packet");
 		g_obex_debug(G_OBEX_DEBUG_ERROR, "%s", (*err)->message);
@@ -1291,7 +1308,9 @@ static gboolean read_packet(GObex *obex, GError **err)
 
 	return TRUE;
 fail:
-	g_obex_debug(G_OBEX_DEBUG_ERROR, "%s", (*err)->message);
+	if (err)
+		g_obex_debug(G_OBEX_DEBUG_ERROR, "%s", (*err)->message);
+
 	return FALSE;
 }
 
@@ -1489,6 +1508,11 @@ GObex *g_obex_ref(GObex *obex)
 	return obex;
 }
 
+static void tx_queue_free(void *data, void *user_data)
+{
+	pending_pkt_free(data);
+}
+
 void g_obex_unref(GObex *obex)
 {
 	int refs;
@@ -1502,7 +1526,7 @@ void g_obex_unref(GObex *obex)
 
 	g_slist_free_full(obex->req_handlers, g_free);
 
-	g_queue_foreach(obex->tx_queue, (GFunc) pending_pkt_free, NULL);
+	g_queue_foreach(obex->tx_queue, tx_queue_free, NULL);
 	g_queue_free(obex->tx_queue);
 
 	if (obex->io != NULL)
@@ -1530,7 +1554,7 @@ void g_obex_unref(GObex *obex)
 /* Higher level functions */
 
 guint g_obex_connect(GObex *obex, GObexResponseFunc func, gpointer user_data,
-					GError **err, guint8 first_hdr_id, ...)
+					GError **err, guint first_hdr_id, ...)
 {
 	GObexPacket *req;
 	struct connect_data data;
@@ -1657,6 +1681,16 @@ guint g_obex_move(GObex *obex, const char *name, const char *dest,
 	return g_obex_send_req(obex, req, -1, func, user_data, err);
 }
 
+guint g_obex_abort(GObex *obex, GObexResponseFunc func, gpointer user_data,
+								GError **err)
+{
+	GObexPacket *req;
+
+	req = g_obex_packet_new(G_OBEX_OP_ABORT, TRUE, G_OBEX_HDR_INVALID);
+
+	return g_obex_send_req(obex, req, -1, func, user_data, err);
+}
+
 guint8 g_obex_errno_to_rsp(int err)
 {
 	switch (err) {
